<?php
/**
 * OOPでBBS (Step8)
 *
 * オブジェクト指向プログラミングで掲示板を作ってみる。
 * ブログでその作り方を晒すために書いたものです。
 * 詳しい事は当該記事を見てください。 
 *
 *
 *           
 * PHP version 5.3
 *
 * LICENSE : このソースはCreative Commons 表示 - 非営利 2.1 日本 Licenseです。
 *           改変は自由ですが営利目的で利用することは出来ません。
 * 
 * @category  Step8
 * @package   OOP-BBS
 * @author    Tenderfeel <tenderfeel@gmail.com>
 * @copyright Copyright (c) 2010 Tenderfeel.(http://webtecnote.com)
 * @license   http://creativecommons.org/licenses/by-nc/2.1/jp/
 * @since     File available since Release 1.0.0 (step1)
 *
 */
 

/**
 * 掲示板をコントロールするクラス
 *
 * ModelとViewの中間に立って仕事を振り分けます。
 * ユーザが起こしたイベントに対して応答し、色々と処理します。
 * 仕分け人なので描画やデータの操作は行いません。
 *
 * <step8>
 * ページネーションつけた。
 * paginateメソッドはページ番号と表示件数からパラメーターを計算、Modelから得たログを返す
 * Viewのpaginateメソッドにパラメーターを渡す
 * Modelのpaginateメソッドに始点と終点を渡す
 * トークンをviewに移した。
 * 
 * 
 * @author    Tenderfeel <tenderfeel@gmail.com>
 * @copyright Copyright (c) 2010 Tenderfeel.(http://webtecnote.com)
 * @license   Creative Commons BY-NC http://creativecommons.org/licenses/by-nc/2.1/jp/
 * @since     File available since Release 1.0.0 (step1)
 */

class Controller {
	
	const INIT_FILE = 'bbs.ini';
	
	public function __construct()
	{
	
		include_once('model.php');
		include_once('view.php');
		include_once('bbs_log.php');
		include_once('config.php');
		
		$iniFilePath = dirname($_SERVER['SCRIPT_FILENAME']).'/lib/'.self::INIT_FILE;
	
		$this->config = new Config($iniFilePath);//設定
		$this->model = new Model($this->config);//モデル
		$this->view = new View($this->config);//ビュー
		
	}
    
	/**
	 * index.phpから実行されるメソッド
	 * @return {}
	 * @since 1.0 (step1)
	 */
	public function execute()
	{
	    $this->model->setLogfileName($this->config->get('logfile_name'));//ログファイルをセット
        $this->model->readLog();//ログを読み込む
		
		if(isset($_POST['mode']) && $_POST['mode'] != ''){
			
			//トークンの確認
			$token = $this->model->clean($_POST['token']);
			
			if($token == @$_COOKIE["_token"]){
				$this->view->setSystemMessage('error', '不正な送信です');//エラーをViewに渡す
				
			}else{
				setcookie("_token", $token);
				$action = $this->model->clean($_POST['mode']).'Action';
				$this->$action();
			}
		}
		
		$viewName = 'index';
		
		if(isset($_GET['view']) && $_GET['view'] != ''){
					
			$viewName = $this->model->clean($_GET['view']);
			
		}
		
		$this->view->setTemplateName($viewName);//テンプレートをセット
			
		$viewName = $viewName.'View';
		
		if(method_exists($this, $viewName)){
			$this->$viewName();//ビュー実行
		}else{
			die('システムエラーが発生しました');
		}
		
	}
    
    
    /**
     * ページネーション
     * @param number $page ページ番号
     * @param number $disp １ページあたりの表示件数
     * @return array ログデータ
     */
    protected function paginate($page, $disp)
    {
        $page = empty($page) ? 1:$page;
        $start = ($page == 1)? 0 : ($page-1) * $disp;
        $end   = ($page * $disp);
        $count = count($this->model->getLogData());
        $pageCount =  floor($count / $disp);
        
        $this->view->paginate(array('page'=>$page, 'pageCount'=>$pageCount, 'dataCount'=>$count, 'limit'=>$disp ));

        return $this->model->paginate($start, $end);
        
    }
	
	/**
	 * 書き込み
	 */
	function writeAction()
	{
		$this->model->setPostdata($_POST);//送信内容のセット

		if($this->model->validation()===true){
			$this->model->write();//ログに書き込み
			$this->view->setSystemMessage('success', '保存しました');//メッセージをセット
            $this->model->readLog();//再読み込み
		}else{
			$this->view->setSystemMessage('error', $this->model->getError());//エラーをViewに渡す
		}
	}
	
	//削除処理
	function deleteAction()
	{
	    $id = $this->model->clean($_POST['id']);
        $log = $this->model->findData($id);
        
        if($log->key === sha1($_POST['key'])){
            if($this->model->delete($id)){
                $this->view->setSystemMessage('success', '削除しました');
                $this->model->readLog();//再読み込み
            }else{
                $this->view->setSystemMessage('error', '削除出来ませんでした');
            }
        }else{
           $this->view->setSystemMessage('error', 'キーが違います');
        }
	}
    
	
	//編集処理
	function editAction()
	{
        $this->model->setPostdata($_POST);//送信内容のセット
        
        if($this->model->validation()===true){
            
            $id = $this->model->clean($_POST['id']);
            $log = $this->model->findData($id);
            
            if($log->key === sha1($_POST['key'])){
                
                    if($this->model->edit($id)){
                        $this->view->setSystemMessage('success', '編集しました');
                        $this->model->readLog();//再読み込み
                    }else{
                        $this->view->setSystemMessage('error', '編集出来ませんでした');
                    }
            }else{
                $this->view->setSystemMessage('error', 'キーが違います');
            }
         
         }else{
             $this->view->setSystemMessage('error', $this->model->getError());//エラーをViewに渡す
         }
	}
	
	//インデックス
	private function indexView()
    {
		$this->view->set('logdata', $this->paginate(@$_GET['page'], 5));//Viewにログを渡す
		$this->view->display();//表示
	}
	
	//編集画面
	private function editView()
	{
		$log = $this->model->findData($this->model->clean($_GET['id']));
		
		$this->view->set('logdata', $log);//Viewにログを渡す
		$this->view->display();//表示
	}
	
	//削除画面
	private function deleteView()
	{
		$log = $this->model->findData($this->model->clean($_GET['id']));
        
		$this->view->set('logdata', $log);//Viewにログを渡す
		$this->view->display();//表示
	}

}
?>